@@ -37,7 +37,7 @@
 #include <QtGui/QPainter>
 #include <QtWidgets/QGraphicsSceneMouseEvent>
 
-QT_CHARTS_BEGIN_NAMESPACE
+QT_BEGIN_NAMESPACE
 
 LineChartItem::LineChartItem(QLineSeries *series, QGraphicsItem *item)
     : XYChart(series,item),
@@ -92,7 +92,7 @@ void LineChartItem::updateGeometry()
     // Store the points to a local variable so that the old line gets properly cleared
     // when animation starts.
     m_linePoints = geometryPoints();
-    const QVector<QPointF> &points = m_linePoints;
+    const QList<QPointF> &points = m_linePoints;
 
     if (points.size() == 0) {
         prepareGeometryChange();
@@ -291,19 +291,8 @@ void LineChartItem::updateGeometry()
         // because shape doesn't get clipped. It doesn't seem possible to do sensibly.
     } else { // not polar
         linePath.moveTo(points.at(0));
-        if (m_pointsVisible) {
-            int size = m_linePen.width();
-            linePath.addEllipse(points.at(0), size, size);
-            linePath.moveTo(points.at(0));
-            for (int i = 1; i < points.size(); i++) {
-                linePath.lineTo(points.at(i));
-                linePath.addEllipse(points.at(i), size, size);
-                linePath.moveTo(points.at(i));
-            }
-        } else {
-            for (int i = 1; i < points.size(); i++)
-                linePath.lineTo(points.at(i));
-        }
+        for (int i = 1; i < points.size(); i++)
+            linePath.lineTo(points.at(i));
         fullPath = linePath;
     }
 
@@ -354,18 +343,24 @@ void LineChartItem::handleUpdated()
     m_pointLabelsVisible = m_series->pointLabelsVisible();
     m_pointLabelsFont = m_series->pointLabelsFont();
     m_pointLabelsColor = m_series->pointLabelsColor();
+    bool labelClippingChanged = m_pointLabelsClipping != m_series->pointLabelsClipping();
     m_pointLabelsClipping = m_series->pointLabelsClipping();
     if (doGeometryUpdate)
         updateGeometry();
     else if (m_series->useOpenGL() && visibleChanged)
         refreshGlChart();
-    update();
+
+    // Update whole chart in case label clipping changed as labels can be outside series area
+    if (labelClippingChanged)
+        m_series->chart()->update();
+    else
+        update();
 }
 
 void LineChartItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
 {
-    Q_UNUSED(widget)
-    Q_UNUSED(option)
+    Q_UNUSED(widget);
+    Q_UNUSED(option);
 
     if (m_series->useOpenGL())
         return;
@@ -401,21 +396,12 @@ void LineChartItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *opt
         painter->setClipRect(clipRect);
     }
 
-    if (m_pointsVisible) {
-        painter->setBrush(m_linePen.color());
+    if (m_linePen.style() != Qt::SolidLine || alwaysUsePath) {
+        // If pen style is not solid line, use path painting to ensure proper pattern continuity
         painter->drawPath(m_linePath);
     } else {
-        painter->setBrush(QBrush(Qt::NoBrush));
-        if (m_linePen.style() != Qt::SolidLine
-                || painter->renderHints().testFlag(QPainter::Antialiasing)
-                || alwaysUsePath) {
-            // If pen style is not solid line, always fall back to path painting
-            // to ensure proper continuity of the pattern
-            painter->drawPath(m_linePath);
-        } else {
-            for (int i(1); i < m_linePoints.size(); i++)
-                painter->drawLine(m_linePoints.at(i - 1), m_linePoints.at(i));
-        }
+        for (int i = 1; i < m_linePoints.size(); ++i)
+            painter->drawLine(m_linePoints.at(i - 1), m_linePoints.at(i));
     }
 
     if (m_pointLabelsVisible) {
@@ -428,6 +414,16 @@ void LineChartItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *opt
 
     painter->restore();
 
+    if (m_pointsVisible) {
+        // draw points that lie inside clipRect only
+        qreal ptSize = m_linePen.width() * 1.5;
+        painter->setPen(Qt::NoPen);
+        painter->setBrush(m_linePen.color());
+        for (int i = 0; i < m_linePoints.size(); ++i) {
+            if (clipRect.contains(m_linePoints.at(i)))
+                painter->drawEllipse(m_linePoints.at(i), ptSize, ptSize);
+        }
+    }
 }
 
 void LineChartItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
@@ -467,6 +463,6 @@ void LineChartItem::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event)
     QGraphicsItem::mouseDoubleClickEvent(event);
 }
 
-QT_CHARTS_END_NAMESPACE
+QT_END_NAMESPACE
 
 #include "moc_linechartitem_p.cpp"
